clear all
set mem 1g
set more off

mkdir c:\EventStudy\S&P500
mkdir c:\EventStudy\S&P500\Input
mkdir c:\EventStudy\S&P500\Temp
mkdir c:\EventStudy\S&P500\Output

capture log close
log using "c:\EventStudy\S&P500\S&P500.log", replace


* --------- Firm Data ------------- *

insheet using "c:\EventStudy\S&P500\Input\02 FirmData.csv", clear delimiter(";")

rename v1 co_id
rename v2 date
rename v3 closing_price_co

replace date = subinstr(date, "-", ".", .)
gen edate = date(date, "YMD")

sort co_id edate
by co_id: gen days_co = _n

xtset co_id days_co
gen co_ret = ln(closing_price/l.closing_price)

sort co_id edate

gen year = substr(date, 1, 4)
destring year, replace
keep if year >= 2003 & year <= 2012

drop if co_ret == .

order co_id year date edate days_co

save "C:\EventStudy\S&P500\Input\FirmData.dta", replace


* --------- Market Data ------------- *

insheet using "c:\EventStudy\S&P500\Input\03 MarketData.csv", clear delimiter(";")

rename v1 mkt_name
rename v2 date
rename v3 closing_price_mkt

replace date = subinstr(date, "-", ".", .) 
gen edate = date(date, "YMD")

sort edate
gen mkt_ret = ln(closing_price[_n]/closing_price[_n-1])

gen year = substr(date, 1, 4)
destring year, replace
keep if year >= 2003 & year <= 2012

drop if mkt_ret == .

order mkt_name year date edate

save "C:\EventStudy\S&P500\Input\MarketData.dta", replace


* --------- Event Data ------------- *

insheet using "c:\EventStudy\S&P500\Input\01 RequestFile.csv", clear delimiter(";")

rename v1 event_id
rename v2 co_id
rename v3 mkt_name
rename v4 date
rename v5 action_type_id
lab var action_type_id "Grouping Variable"
rename v6 evnt_window_start
rename v7 evnt_window_end
rename v8 est_window_endpointer
rename v9 est_window_length

replace date = subinstr(date, "-", ".", .) 
gen edate = date(date, "YMD")

sort edate

gen year = substr(date, 1, 4)
destring year, replace
keep if year >= 2003 & year <= 2012

order event_id co_id mkt_name year date edate

save "C:\EventStudy\S&P500\Input\EventData.dta", replace


* --------- Combining Firm, Market & Event Data ------------- *

sort co_id edate
merge co_id edate using "C:\EventStudy\S&P500\Input\FirmData.dta"
drop _merge

sort edate
merge edate using "C:\EventStudy\S&P500\Input\MarketData.dta"

* Note: _merge = 1 for some observations, implying presence of non-trading event days
drop _merge

sort co_id edate

save "C:\EventStudy\S&P500\Input\Data.dta", replace


* --------- Combining Firm Industry Classification information --------- *

use "C:\EventStudy\S&P500\Input\02 Firm-Industry Classification.dta", clear
rename firm_id co_id
rename firm_name co_name
sort co_id
save "C:\EventStudy\S&P500\Input\Industry Classification.dta", replace

use "C:\EventStudy\S&P500\Input\Data.dta", clear
sort co_id
merge co_id using "C:\EventStudy\S&P500\Input\Industry Classification.dta"
drop _m
save "C:\EventStudy\S&P500\Input\Data.dta", replace


* --------- Combining Firm Financials --------- *

* 1> Profits 
* --------- *

use "C:\EventStudy\S&P500\Input\04_Firm_Profits.dta", clear
rename firm_id co_id
* Checking for Duplicates
* duplicates tag co_id, gen(dup_co)
* br if dup_co != 0
* Checked - All duplicates in terms of co_id have identical Profit figures for 2003-2012
* drop dup_co
duplicates drop co_id, force
forvalues i = 2003/2012 {
	  rename _`i' profit`i'
	  }
reshape long profit, i(co_id) j(year)
sort co_id year
save "C:\EventStudy\S&P500\Input\Profits.dta", replace

use "C:\EventStudy\S&P500\Input\Data.dta", clear
sort co_id year
merge co_id year using "C:\EventStudy\S&P500\Input\Profits.dta"
drop if _m == 2
drop _m
save "C:\EventStudy\S&P500\Input\Data.dta", replace


* 2> Sales
* --------- *

use "C:\EventStudy\S&P500\Input\05_Firm_Sales.dta", clear
rename firm_id co_id
* Checking for Duplicates
* duplicates tag co_id, gen(dup_co)
* br if dup_co != 0
* Checked - All duplicates in terms of co_id have identical Sales figures for 2003-2012
* drop dup_co
duplicates drop co_id, force
forvalues i = 2003/2012 {
	  rename _`i' sales`i'
	  }
reshape long sales, i(co_id) j(year)
sort co_id year
save "C:\EventStudy\S&P500\Input\Sales.dta", replace

use "C:\EventStudy\S&P500\Input\Data.dta", clear
sort co_id year
merge co_id year using "C:\EventStudy\S&P500\Input\Sales.dta"
drop if _m == 2
drop _m
save "C:\EventStudy\S&P500\Input\Data.dta", replace


* --------- Combining Action Type & Action Domain information --------- *

use C:\EventStudy\S&P500\Input\06_Types2Domains.dta, clear
rename c1 action_type
rename c2 action_domain
drop if _n == 1
sort action_type
save "C:\EventStudy\S&P500\Input\ActionTypes&Domains.dta", replace

use "C:\EventStudy\S&P500\Input\01 Firm Competitive Actions.dta", clear
keep if year >= 2003 & year <= 2012
rename firm_id co_id
rename category_id action_type_id
rename category_label action_type
sort action_type
save "C:\EventStudy\S&P500\Input\CompetitiveActions.dta", replace

merge action_type using "C:\EventStudy\S&P500\Input\ActionTypes&Domains.dta"
drop if _m == 2
drop _m
drop if date == .
replace action_domain = "Communication and Reports" if action_type == "Corporate scandal"
drop if action_type == ""
rename date edate
sort co_id edate
save "C:\EventStudy\S&P500\Input\CompetitiveActions.dta", replace

use "C:\EventStudy\S&P500\Input\Data.dta", clear
sort co_id edate
merge co_id edate using "C:\EventStudy\S&P500\Input\CompetitiveActions.dta"
drop _m
save "C:\EventStudy\S&P500\Input\Data.dta", replace


* --------- Cleaning the Dataset --------- *

* Flagging Trading Days & Non-trading Event Days
* ---------------------------------------------- *
gen trd_day = 1 if co_ret != . | mkt_ret != .
gen nontrd_evntday = 1 if trd_day != 1 & event_id != .

* Ignore Events on trading days with non-missing market returns, but missing firm returns
* --------------------------------------------------------------------------------------- *
replace event_id = . if event_id != . & trd_day == 1 & co_ret == .


* Note: There are no observations on some companies
* Creating a continuous identifier
* ------------------------------------------------- *
* summ co_id
* forvalues i = 1/`r(max)' {
*	    count if co_id == `i'
*	    }
egen co_id_cont = group(co_id)

* Note: Multiple events for almost all companies & multiple events on a single day for some companies
* Dropping companies with no events
* --------------------------------------------------------------------------------------------------- *
gen event = 1 if event_id != .
bysort co_id_cont: gen event_no = sum(event) if event == 1

quietly summ co_id_cont
local tot_cos = r(max)
gen zero_evnt = .
forvalues i = 1/`tot_cos' {

	  quietly summ event_no if co_id_cont == `i'
	  local tot_evnt_co`i' = r(max)

	  replace zero_evnt = 1 if co_id_cont == `i' & `tot_evnt_co`i'' == .

	  }

drop if zero_evnt == 1
drop zero_evnt

* Creating a continuous identifier
* -------------------------------- *
drop co_id_cont
egen co_id_2 = group(co_id)


* Adjustment for Non-trading Event Days
* All adjustments are made when the same set of commands is run 59 times
* ---------------------------------------------------------------------- *

sort co_id_2 edate

gen no_adjust = 1 if nontrd_evntday[_n] == 1 & co_id_2[_n+1] != co_id_2[_n]
drop if no_adjust == 1
* Note: No such observations, so all 6649 non-trading event days can be adjusted
drop no_adjust


forvalues i = 1/59 {

replace date = date[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1
replace edate = edate[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1
replace days_co = days_co[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1

replace closing_price_co = closing_price_co[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1
replace co_ret = co_ret[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1

replace closing_price_mkt = closing_price_mkt[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1
replace mkt_ret = mkt_ret[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1

replace trd_day = trd_day[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1
replace nontrd_evntday = nontrd_evntday[_n+1] if co_id_2[_n+1] == co_id_2[_n] & nontrd_evntday[_n] == 1 & trd_day[_n+1] == 1

count if nontrd_evntday == 1

}

drop nontrd_evntday


* Post-adjustment for non-trading event days with returns & date info of the following trading day, we have some more events with missing firm returns on event day
* This is because the following trading day with missing firm return may not have been an event day, and so was not filtered by * replace event_id = . if event_id != . & trd_day == 1 & co_ret == . *
* Ignore such events on trading days with missing firm returns
* ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- *
replace event_id = . if event_id != . & co_ret == .

* --------- Dropping companies with no events --------- *
drop event event_no
gen event = 1 if event_id != .
bysort co_id_2: gen event_no = sum(event) if event == 1

quietly summ co_id_2
local tot_cos = r(max)
gen zero_evnt = .
forvalues i = 1/`tot_cos' {

	  quietly summ event_no if co_id_2 == `i'
	  local tot_evnt_co`i' = r(max)

	  replace zero_evnt = 1 if co_id_2 == `i' & `tot_evnt_co`i'' == .

	  }

drop if zero_evnt == 1
drop zero_evnt

* Creating a continuous identifier
* -------------------------------- *
drop co_id_2
egen co_id_3 = group(co_id)

* Flagging multiple events on the same day (to help set Estimation & Event Windows - as below )
* --------------------------------------------------------------------------------------------- *
duplicates tag co_id edate, gen(dup_co_dt)
replace dup_co_dt = 1 if dup_co_dt != 0
bysort co_id edate: gen multi_evnt = sum(dup_co_dt) if dup_co_dt == 1
replace multi_evnt = . if multi_evnt == 1
drop dup_co_dt


save "C:\EventStudy\S&P500\Input\Data.dta", replace


* --------- Setting Estimation & Event Windows ------------- *
* Ignoring events lacking the required minimum no. of observations for Estimation & Event Windows
* Splitting the data per Event, tagged with corresponding Firm ID (co_id_3 & event_no)  
* ----------------------------------------------------------------------------------------------- *

quietly summ est_window_length
local estw_length = r(mean)

quietly summ est_window_endpointer
local estw_endpointer = r(mean) + 1

quietly summ evnt_window_start
local evntw_start = r(mean)

quietly summ evnt_window_end 
local evntw_end = r(mean)

local evntw_length = `evntw_end' - `evntw_start' + 1

quietly summ co_id_3
local tot_cos = r(max)


local v = 1
forvalues i = 1/`tot_cos' {

	  use "C:\EventStudy\S&P500\Input\Data.dta", clear
	  keep if co_id_3 == `i'
	  save "C:\EventStudy\S&P500\Temp\Data_Co`i'.dta", replace

	  quietly summ event_no
	  local tot_evnt = r(max)

	  forvalues j = 1/`tot_evnt' {

		    use "C:\EventStudy\S&P500\Temp\Data_Co`i'.dta", clear

		    quietly summ days_co if event_no == `j'
		    local evntday`j' = r(mean)

		    gen estw_evnt`j' = 1 if (`evntday`j'' + `estw_endpointer' - days_co) <= `estw_length' & co_ret != . & mkt_ret != . & multi_evnt == .
		    replace estw_evnt`j' = . if (`evntday`j'' + `estw_endpointer' - days_co) <= 0

		    gen evntw_evnt`j' = 1 if (days_co - `evntday`j'') >= `evntw_start' & (days_co - `evntday`j'') <= `evntw_end' & co_ret != . & mkt_ret != . & multi_evnt == .

		    gen evnt`j'_estw_day = sum(estw_evnt`j') if estw_evnt`j' == 1
		    quietly summ evnt`j'_estw_day
		    local evnt`j'_estw_length = r(max)

		    gen evnt`j'_evntw_day = sum(evntw_evnt`j') if evntw_evnt`j' == 1
		    quietly summ evnt`j'_evntw_day
		    local evnt`j'_evntw_length = r(max)

		    replace event_id = . if event_id != . & event_no == `j' & `evnt`j'_estw_length' < `estw_length' | event_id != . & event_no == `j' & `evnt`j'_evntw_length' < `evntw_length' | event_id != . & event_no == `j' & `evnt`j'_estw_length' == . | event_id != . & event_no == `j' & `evnt`j'_evntw_length' == .

		    gen keep_event = 1 if event_id != . & event_no == `j'
		    quietly summ keep_event if event_no == `j'
		    local keep_event`j'_co`i' = r(mean)

		    
		    if `keep_event`j'_co`i'' == 1 {

		       keep if estw_evnt`j' == 1 | evntw_evnt`j' == 1

		       rename estw_evnt`j' estw
		       rename evntw_evnt`j' evntw
		       rename evnt`j'_estw_day estw_day
		       rename evnt`j'_evntw_day evntw_day

		       gen event_day = 1 if days_co == `evntday`j''

		       replace event_id = . if event_day != 1
		       replace mkt_name = "SP500"
		       replace action_type_id = . if event_day != 1
		       replace evnt_window_start = . if event_day != 1
		       replace evnt_window_end = . if event_day != 1
		       replace est_window_endpointer = . if event_day != 1
		       replace est_window_length = . if event_day != 1
		       replace text_id = . if event_day != 1
		       replace action_type = "" if event_day != 1
		       replace action_domain = "" if event_day != 1
		       replace event_no = . if event_day != 1

		       drop event keep_event

		       order co_id co_id_3 co_name ds_code thomson_ticker sic_code_1-sic_code_36 profit sales year date edate days_co trd_day multi_evnt closing_price_co co_ret mkt_name closing_price_mkt mkt_ret event_no event_id text_id action_type_id action_type action_domain est_window_endpointer est_window_length evnt_window_start evnt_window_end estw estw_day evntw evntw_day
		       
		       save "C:\EventStudy\S&P500\Temp\Co`i'_Event`j'.dta", replace


		       quietly summ event_no if event_day == 1

		       if r(mean) != `j' {

			  use "C:\EventStudy\S&P500\Temp\Data_Co`i'.dta", clear
			  keep if event_no == `j'
			  sort days_co
			  save "C:\EventStudy\S&P500\Temp\Co`i'_Evnt`j'Day.dta", replace

			  use "C:\EventStudy\S&P500\Temp\Co`i'_Event`j'.dta", clear
			  sort days_co
			  merge days_co using "C:\EventStudy\S&P500\Temp\Co`i'_Evnt`j'Day.dta", update replace
			  drop _m
			  save "C:\EventStudy\S&P500\Temp\Co`i'_Event`j'.dta", replace

			  }


* --------- Estimating Normal Performance, Calculating Abnormal Returns & Testing for Significance ------------- *

			reg co_ret mkt_ret if estw == 1
			local rmse = e(rmse)
			predict p if evntw == 1
			rename p predret

			gen ar = co_ret - predret if evntw == 1

			save "C:\EventStudy\S&P500\Temp\Co`i'_Event`j'.dta", replace


			local k = `evntw_end' - `evntw_start' + 1
			forvalues l = 1/`k' {

		  		  quietly summ ar if evntw_day == `l'
		  		  gen ar_day`l' = r(mean) if event_day == 1

		  		  gen tvalue_ar_day`l' = ar_day`l'/`rmse' if event_day == 1

		  		  }

			drop ar
			keep if event_day == 1

			egen car = rowtotal(ar*)

			gen tvalue_car = car/(sqrt(`evntw_end' - `evntw_start' + 1) * `rmse')


			save "C:\EventStudy\S&P500\Temp\Event`v'.dta", replace

			local v = `v' + 1		       

		       }

		    }

	  }


* --------- Stacking all Events ------------- *

local i = `v' - 1
use "C:\EventStudy\S&P500\Temp\Event1.dta", clear
forvalues j = 2/`i' {
	  append using "C:\EventStudy\S&P500\Temp\Event`j'.dta"
	  }
save "C:\EventStudy\S&P500\Output\Events.dta", replace

	  

log close

